@@ -4,8 +4,8 @@
 
 ;; Author: Phil Hagelberg
 ;; URL: https://git.sr.ht/~technomancy/fennel-mode
-;; Package-Version: 0.9.2
-;; Package-Revision: 0.9.2-0-g14c853c1bef5
+;; Package-Version: 20251025.155
+;; Package-Revision: e903eb51980a
 ;; Created: 2018-02-18
 ;; Package-Requires: ((emacs "26.1"))
 ;; Keywords: languages, tools
@@ -59,20 +59,41 @@
   :type 'string
   :package-version '(fennel-mode "0.3.0"))
 
-(defcustom fennel-mode-annotate-completion t
-  "Whether or not to show kind of completion candidates."
-  :group 'fennel-mode
-  :type 'boolean
-  :package-version '(fennel-mode "0.4.0"))
+;; TODO: remove all obsolete variables in the 1.0.0 release
+(define-obsolete-variable-alias
+  'fennel-mode-annotate-completion
+  'fennel-proto-repl-annotate-completion
+  "fennel-mode 0.9.3"
+  "Completion annotation is now a feature of `fennel-proto-repl' mode.
+This variable will be removed in the 1.0.0 release of `fennel-mode'.")
 
-(defvaralias 'fennel-mode-repl-prompt-regexp
-  'fennel-mode-repl-prompt)
+(define-obsolete-variable-alias
+  'fennel-eldoc-fontify-markdown
+  'fennel-proto-repl-eldoc-fontify-markdown
+  "fennel-mode 0.7.0"
+  "Eldoc is now a feature of `fennel-proto-repl' mode.
+This variable will be removed in the 1.0.0 release of `fennel-mode'.")
 
-(make-obsolete-variable
- 'fennel-mode-repl-prompt-regexp
- 'fennel-mode-repl-prompt
- "fennel-mode 0.8.0"
- 'set)
+(define-obsolete-variable-alias
+  'fennel-mode-repl-prompt-regexp
+  'fennel-mode-repl-prompt
+  "fennel-mode 0.8.0"
+  "This variable will be removed in the 1.0.0 release of `fennel-mode'.")
+
+(define-obsolete-variable-alias
+  'fennel-mode-repl-subprompt-regexp
+  'fennel-mode-repl-subprompt
+  "fennel-mode 0.8.0"
+  "This variable will be removed in the 1.0.0 release of `fennel-mode'.")
+
+(define-obsolete-variable-alias
+  'fennel-mode-repl-log-communications
+  'fennel-proto-repl-log-communication
+  "fennel-mode 0.8.0"
+  "Logging is now a feature of `fennel-proto-repl' mode.
+This variable will be removed in the 1.0.0 release of `fennel-mode'.")
+
+(make-obsolete-variable 'fennel-repl-minify-code nil "fennel-mode 0.7.0")
 
 (defcustom fennel-mode-repl-prompt ">>"
   "String that matches REPL prompt.
@@ -84,15 +105,6 @@ expression to match both prompt and subpromt."
          (set-default-toplevel-value sym (string-trim-right val)))
   :package-version '(fennel-mode "0.8.0"))
 
-(defvaralias 'fennel-mode-repl-subprompt-regexp
-  'fennel-mode-repl-subprompt)
-
-(make-obsolete-variable
- 'fennel-mode-repl-subprompt-regexp
- 'fennel-mode-repl-subprompt
- "fennel-mode 0.8.0"
- 'set)
-
 (defcustom fennel-mode-repl-subprompt ".."
   "String that matches REPL subprompt.
 The string is automatically escaped and used as a regular
@@ -109,17 +121,6 @@ expression to match both prompt and subpromt."
   :type 'boolean
   :package-version '(fennel-mode "0.6.0"))
 
-(defcustom fennel-mode-repl-log-communications nil
-  "Whether to log REPL communication to a hidden *fennel-repl-log* buffer."
-  :group 'fennel-mode
-  :type 'boolean
-  :package-version '(fennel-mode "0.6.0"))
-
-(make-obsolete-variable
- 'fennel-mode-repl-log-communications
- "logging is now a feature of fennel-proto-repl."
- "fennel-mode 0.8.0")
-
 (defcustom fennel-view-compilation-mode
   (cond ((fboundp 'lua-mode) #'lua-mode)
         ((fboundp 'lua-ts-mode) #'lua-ts-mode)
@@ -238,9 +239,11 @@ specified function F."
     (,(regexp-opt fennel-keywords 'symbols) . font-lock-keyword-face)
     (,(regexp-opt fennel-builtin-functions 'symbols) . font-lock-builtin-face)
     (,(rx bow "$" (optional digit) eow) . font-lock-keyword-face)
-    (,(rx (group ":" (1+ word))) 0 font-lock-builtin-face)
-    (,(rx (group letter (0+ word) "." (1+ word))) 0 font-lock-type-face)
-    (,(rx bow "&" (optional "as") eow) . font-lock-keyword-face)))
+    (,(rx bow (group ":" (1+ word))) 0 font-lock-type-face)
+    (,(rx bow "&" (optional "as") eow) . font-lock-keyword-face)
+    (,(rx "`" (group-n 1 (optional "#'")
+                       (+ (or (syntax symbol) (syntax word)))) "`")
+     (1 'font-lock-constant-face prepend))))
 
 (defvar fennel-doc-string-elt-property 'doc-string-elt
   "The symbol property that holds the doc string position info.")
@@ -406,28 +409,8 @@ If ASK? or LAST-MODULE were not supplied, asks for the name of a module."
   (let ((module (if (or ask? (not last-module))
                     (read-string "Module: " (or last-module (file-name-base nil)) 'fennel-mode-module-names)
                   last-module)))
-    (setq fennel-module-name module)    ; remember for next time
-    (intern (concat ":" module))))
-
-(defun fennel-reload-form (module-keyword)
-  "Return a string of the code to reload the MODULE-KEYWORD module."
-  (format "%s\n" `(let [old (require ,module-keyword)
-                            _ (tset package.loaded ,module-keyword nil)
-                            (ok new) (pcall require ,module-keyword)
-                            ;; keep the old module if reload failed
-                            new (if (not ok) (do (print new) old) new)]
-                    ;; if the module isn't a table then we can't make
-                    ;; changes which affect already-loaded code, but if
-                    ;; it is then we should splice new values into the
-                    ;; existing table and remove values that are gone.
-                    (when (and (= (type old) :table) (= (type new) :table))
-                      (each [k v (pairs new)]
-                            (tset old k v))
-                      (each [k (pairs old)]
-                            ;; the elisp reader is picky about where . can be
-                            (when (= nil ("." new k))
-                              (tset old k nil)))
-                      (tset package.loaded ,module-keyword old)))))
+    (setq fennel-module-name module)    ; remember for the next reload
+    module))
 
 (defun fennel-reload (ask?)
   "Reload the module for the current file.
@@ -448,35 +431,10 @@ buffer, or when given a prefix arg."
                (yes-or-no-p "Lua file for module exists; delete it first?"))
       (delete-file (concat (file-name-base nil) ".lua"))))
   (let ((module (fennel-get-module ask? fennel-module-name)))
-    (comint-send-string (inferior-lisp-proc) (fennel-reload-form module)))
+    (comint-send-string (inferior-lisp-proc) (format ",reload %s\n" module)))
   (when fennel-mode-switch-to-repl-after-reload
     (switch-to-lisp t)))
 
-(defun fennel-completion--candidate-kind (item)
-  "Annotate completion kind of the ITEM for company-mode."
-  (when fennel-mode-annotate-completion
-    (let ((type (when-let* ((buf (fennel-repl-redirect-one
-                                  (inferior-lisp-proc)
-                                  (format "(print (type %s))" item)
-                                  " *fennel-completion-annotation*")))
-                  (with-current-buffer buf
-                    (let ((contents (buffer-substring-no-properties (point-min) (point-max))))
-                      (string-trim (ansi-color-apply contents)))))))
-      (cond ((string= type "function") 'function)
-            ((string= type "table") 'module)
-            ((and type (string-match-p "tried to reference a macro" type)) 'macro)
-            ((and type (string-match-p "tried to reference a special form" type)) 'keyword)
-            ((string-match-p "\\." item) 'field)
-            (t 'variable)))))
-
-(defun fennel-completion--annotate (item)
-  "Annotate completion kind of the ITEM."
-  (let ((kind (fennel-completion--candidate-kind item)))
-    (cond ((eq kind 'module) " table")
-          ((eq kind 'variable) " definition")
-          (kind (format " %s" kind))
-          (t ""))))
-
 (defun fennel-completions (input)
   "Query completions for the INPUT from the `inferior-lisp-proc'."
   (condition-case nil
@@ -501,26 +459,7 @@ Requires Fennel 0.9.3+."
     (when completions
       (list (car bounds)
             (cdr bounds)
-            completions
-            :annotation-function #'fennel-completion--annotate
-            :company-kind #'fennel-completion--candidate-kind))))
-
-(defun fennel-find-definition-for (identifier)
-  "Find the definition of IDENTIFIER."
-  (let ((tempfile (make-temp-file "fennel-find-")))
-    (comint-send-string
-     (inferior-lisp-proc)
-     (format "%s\n"
-             `(with-open [f (io.open ,(format "\"%s\"" tempfile) :w)]
-                         (match (-?> ,identifier (debug.getinfo))
-                                {:what :Lua
-                                : source : linedefined} (f:write source :! linedefined)))))
-    (sit-for 0.1)
-    (when (file-exists-p tempfile)
-      (with-temp-buffer
-        (insert-file-contents tempfile)
-        (delete-file tempfile)
-        (buffer-substring-no-properties (point-min) (point-max))))))
+            completions))))
 
 ;;; xref
 
@@ -532,10 +471,25 @@ Requires Fennel 0.9.3+."
   "Return the relevant identifier at point."
   (thing-at-point 'symbol))
 
+(defun fennel-mode--find-definition-for (identifier)
+  "Find the definition of IDENTIFIER."
+  (let* ((command (format ",find %s\n" identifier))
+         (proc (inferior-lisp-proc))
+         (buf (fennel-repl-redirect-one proc command " *fennel-find*")))
+    (when buf
+      (with-current-buffer buf
+        (buffer-substring-no-properties (point-min) (point-max))))))
+
+(define-obsolete-function-alias
+  'fennel-find-definition-for
+  'fennel-mode--find-definition-for
+  "fennel-mode 0.9.3"
+  "This functionality was moved to `xref'.")
+
 (cl-defmethod xref-backend-definitions ((_backend (eql fennel)) identifier)
   "Find definitions of IDENTIFIER."
-  (let ((location (fennel-find-definition-for identifier)))
-    (when (string-match "^@?\\(.+\\)!\\(.+\\)" location)
+  (let ((location (fennel-mode--find-definition-for identifier)))
+    (when (string-match "^@?\\([^:]+\\):\\([[:digit:]]+\\)" location)
       (let ((file (match-string 1 location))
             (line (1+ (string-to-number (match-string 2 location)))))
         (message "found %s %s" file line)
@@ -544,6 +498,7 @@ Requires Fennel 0.9.3+."
 (defvar fennel-module-history nil)
 (defvar fennel-field-history nil)
 
+;; TODO: remove/obsolete?
 (defun fennel-find-module-field (module fields-string)
   "Find FIELDS-STRING in the MODULE."
   (let ((tempfile (make-temp-file "fennel-module-"))
@@ -617,11 +572,18 @@ variable."
 (defalias 'fennel-eval-form-and-next 'lisp-eval-form-and-next)
 (defalias 'fennel-eval-region 'lisp-eval-region)
 
+(defun fennel-format-replace ()
+  (when (< 0 (shell-command-on-region (point-min) (point-max) "fnlfmt -"
+                                      "*fennel-format*" nil
+                                      "*fennel-error*" t))
+    (error "could not format %s" (current-buffer)))
+  (get-buffer "*fennel-format*"))
+
 (defun fennel-format-region (start end)
   "Run fnlfmt on the region from START to END."
   (interactive "r")
   (if (executable-find "fnlfmt")
-      (shell-command-on-region start end "fnlfmt -" nil t)
+      (replace-region-contents start end 'fennel-format-replace)
     (message "fnlfmt not found")))
 
 (defun fennel-format ()
@@ -670,6 +632,8 @@ variable."
 ;;;###autoload
 (add-to-list 'auto-mode-alist '("\\.fnl\\'" . fennel-mode))
 ;;;###autoload
+(add-to-list 'auto-mode-alist '("\\.fnlm\\'" . fennel-mode))
+;;;###autoload
 (add-to-list 'interpreter-mode-alist '("fennel" . fennel-mode))
 
 ;;; REPL support
@@ -678,35 +642,6 @@ variable."
 (defvar fennel-repl--buffer nil)
 (defvar fennel-repl--last-buffer nil)
 
-(defcustom fennel-repl-minify-code nil
-  "How to minify code before sending it to the process.
-
-The default value of `oneline' should not be changed unless there
-are some misbehaviors when interacting with the REPL.
-
-Possible variants:
-
-- \\'strings - collapse strings with literal newlines into
-  one-line strings with `\\n'.
-- \\'comments - remove comments, and lines completely consisting
-  out of comments.
-- \\'both - collapse strings and remove comments.
-- \\'oneline - collapse strings, remove comments, and remove any
-  newlines from the code.
-- nil - don't modify anything."
-  :group 'fennel-mode
-  :type '(choice
-	  (const :tag "collapse strings" strings)
-	  (const :tag "remove comments" comments)
-	  (const :tag "collapse strings and remove comments" both)
-          (const :tag "collapse to one line" oneline)
-          (const :tag "do not change" nil)))
-
-(make-obsolete-variable
- 'fennel-repl-minify-code
- "minification is no longer preformed; variable value is ignored."
- "fennel-mode 0.7.0")
-
 (defun fennel-repl--input-filter (body)
   "Input filter for the comint process.
 Checks if BODY is a balanced expression."
